@@ -1,4 +1,4 @@
 setMethod("reorder", signature(x = "phylo"), function(x, order = 'cladewise') {
-    x <- reorder.phylo(x,  order)
+    x <- ape::reorder.phylo(x,  order)
     x
 })
